﻿// Kerbal Engineer Redux
// Version: 0.1
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System.Collections.Generic;
using UnityEngine;

namespace Engineer
{
    class Engineer : Part
    {
        // Initialise global variables.
        GUIStyle guiStyle;
        Rect winPos = new Rect(300, 70, 0, 0);
        List<Part> partList;
        int stageCount = 0;

        // Event triggered when part is attached to ship in VAB.
        protected override void onPartAttach(Part parent)
        {
            this.manualStageOffset = 0;
            RenderingManager.AddToPostDrawQueue(0, DrawGUI);
            base.onPartAttach(parent);
        }

        // Event triggered when part is detached from ship in VAB.
        protected override void onPartDetach()
        {
            RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);
            base.onPartDetach();
        }

        // Method containing the routines to draw the GUI.
        public void DrawGUI()
        {
            GUI.skin = HighLogic.Skin;
            winPos = GUILayout.Window(1, winPos, MainGUI, "Kerbal Engineer Redux - Version 0.1 ", GUILayout.MinWidth(750));
        }

        public void MainGUI(int windowId)
        {
            // Get all attached ship parts.
            partList = EditorLogic.SortedShipList;

            guiStyle = new GUIStyle(GUI.skin.label);
            guiStyle.normal.textColor = Color.white;
            GUI.skin = HighLogic.Skin;

            GUILayout.BeginHorizontal();

            // STAGES
            GUILayout.BeginVertical();
            GUILayout.Label("");
            stageCount = GetStageCount();
            for (int i = 0; i <= stageCount; i++)
                GUILayout.Label("S " + i.ToString());
            GUILayout.EndVertical();

            // MASS / TOTAL MASS
            GUILayout.BeginVertical();
            GUILayout.Label("Mass", guiStyle);
            for (int i = 0; i <= stageCount; i++)
                GUILayout.Label(GetStageMass(i).ToString("0.#") + " / " + GetStageTotalMass(i).ToString("0.#"));
            GUILayout.EndVertical();

            // FUEL
            GUILayout.BeginVertical();
            GUILayout.Label("Fuel", guiStyle);
            for (int i = 0; i <= stageCount; i++)
                GUILayout.Label(GetStageFuel(i).ToString("0.##"));
            GUILayout.EndVertical();

            // ATMOSPHERIC ISP
            GUILayout.BeginVertical();
            GUILayout.Label("a-Isp", guiStyle);
            for (int i = 0; i <= stageCount; i++)
                GUILayout.Label(GetStageAtmsIsp(i).ToString("0.#"));
            GUILayout.EndVertical();

            // VACUUM ISP
            GUILayout.BeginVertical();
            GUILayout.Label("v-Isp", guiStyle);
            for (int i = 0; i <= stageCount; i++)
                GUILayout.Label(GetStageVacIsp(i).ToString("0.#"));
            GUILayout.EndVertical();
            
            // THRUST
            GUILayout.BeginVertical();
            GUILayout.Label("Thrust", guiStyle);
            for (int i = 0; i <= stageCount; i++)
                GUILayout.Label(GetStageThrust(i).ToString());
            GUILayout.EndVertical();
            
            // THRUST TO WEIGHT - KERBIN
            GUILayout.BeginVertical();
            GUILayout.Label("Kerbin", guiStyle);
            for (int i = 0; i <= stageCount; i++)
                GUILayout.Label(GetStageThrustToWeight(i, 9.8068f).ToString("0.##"));
            GUILayout.EndVertical();

            // THRUST TO WEIGHT - MUN
            GUILayout.BeginVertical();
            GUILayout.Label("Mun", guiStyle);
            for (int i = 0; i <= stageCount; i++)
                GUILayout.Label(GetStageThrustToWeight(i, 1.6284f).ToString("0.##"));
            GUILayout.EndVertical();

            // THRUST TO WEIGHT - MINMUS
            GUILayout.BeginVertical();
            GUILayout.Label("Minmus", guiStyle);
            for (int i = 0; i <= stageCount; i++)
                GUILayout.Label(GetStageThrustToWeight(i, 0.785f).ToString("0.##"));
            GUILayout.EndVertical();

            // DELTA-V
            GUILayout.BeginVertical();
            GUILayout.Label("Delta-V", guiStyle);
            for (int i = 0; i <= stageCount; i++)
                GUILayout.Label(GetStageDeltaV(i).ToString("0") + "m/s");
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();

            // Allow window to be moved.
            GUI.DragWindow(new Rect(0, 0, winPos.width, winPos.height));
        }

        // Get the number of stages.
        public int GetStageCount()
        {
            int stageCount = 0;

            foreach (Part part in partList)
                if (part.inverseStage > stageCount)
                    stageCount = part.inverseStage;

            return stageCount;
        }

        // Get the MASS of the current stage.
        public float GetStageMass(int stage)
        {
            float totalMass = 0.0f;

            foreach (Part part in partList)
                if (part.inverseStage == stage)
                    totalMass += part.mass;

            return totalMass;
        }

        // Get the MASS from the current stage upwards.
        public float GetStageTotalMass(int stage)
        {
            float totalMass = 0.0f;

            for (int i = 0; i <= stage; i++)
                totalMass += GetStageMass(i);

            return totalMass;
        }

        // Get the DRY MASS of stage. (Used Fuel)
        public float GetStageDryMass(int stage)
        {
            float totalDryMass = 0;

            foreach (Part part in partList)
                if (part.inverseStage == stage)
                {
                    if (part is FuelTank)
                        totalDryMass += (part as FuelTank).dryMass;
                }

            return totalDryMass;
        }

        // Get the FUEL MASS of stage.
        public float GetStageFuelMass(int stage)
        {
            float totalFuelMass = 0;

            foreach (Part part in partList)
                if (part.inverseStage == stage)
                {
                    if (part is FuelTank)
                        totalFuelMass += (part as FuelTank).mass - (part as FuelTank).dryMass;
                }

            return totalFuelMass;
        }

        // Get the FUEL amount of stage.
        public float GetStageFuel(int stage)
        {
            float totalFuel = 0;

            foreach (Part part in partList)
                if (part.inverseStage == stage)
                {
                    if (part is FuelTank)
                        totalFuel += (part as FuelTank).fuel;
                }

            return totalFuel;
        }

        // Get the ATMOSPHERIC ISP of stage.
        public float GetStageAtmsIsp(int stage)
        {
            float totalIsp = 0;
            int parts = 0;

            foreach (Part part in partList)
                if (part.inverseStage == stage)
                {
                    if (part is LiquidFuelEngine)
                    {
                        totalIsp += (part as LiquidFuelEngine).Isp;
                        parts++;
                    }
                    if (part is AtmosphericEngine)
                    {
                        totalIsp += (part as AtmosphericEngine).airflowEfficiency;
                        parts++;
                    }
                }

            totalIsp = totalIsp / parts;

            if (totalIsp / parts > 0)
                return totalIsp;
            else
                return 0;
        }

        // Get the VACUUM ISP of stage.
        public float GetStageVacIsp(int stage)
        {
            float totalIsp = 0;
            int parts = 0;

            foreach (Part part in partList)
                if (part.inverseStage == stage && (part is LiquidFuelEngine))
                {
                    totalIsp += (part as LiquidFuelEngine).vacIsp;
                    parts++;
                }

            totalIsp = totalIsp / parts;

            if (totalIsp / parts > 0)
                return totalIsp;
            else
                return 0;
        }

        // Get the THRUST of stage.
        public float GetStageThrust(int stage)
        {
            float totalThrust = 0;

            foreach (Part part in partList)
                if (part.inverseStage == stage)
                {
                    if (part is AtmosphericEngine)
                        totalThrust += (part as AtmosphericEngine).maximumEnginePower;
                    if (part is LiquidEngine)
                        totalThrust += (part as LiquidEngine).maxThrust;
                    if (part is LiquidFuelEngine)
                        totalThrust += (part as LiquidFuelEngine).maxThrust;
                    if (part is SolidRocket)
                        totalThrust += (part as SolidRocket).thrust;
                }

            return totalThrust;
        }

        // Get the THRUST TO WEIGHT ratio of stage based on gravity input.
        public float GetStageThrustToWeight(int stage, float gravity)
        {
            return (GetStageThrust(stage) / GetStageTotalMass(stage)) / gravity;
        }

        // Get the FUEL CONSUMPTION of stage.
        public float GetStageFuelConsumption(int stage)
        {
            float totalFuelConsumption = 0;

            foreach (Part part in partList)
                if (part.inverseStage == stage)
                {
                    if (part is AtmosphericEngine)
                        totalFuelConsumption += (part as AtmosphericEngine).fuelConsumption;
                    if (part is LiquidEngine)
                        totalFuelConsumption += (part as LiquidEngine).fuelConsumption;
                    if (part is LiquidFuelEngine)
                        totalFuelConsumption += (part as LiquidFuelEngine).fuelConsumption;
                    //if (part is SolidRocket)
                        //totalFuelConsumption += (part as SolidRocket).fuelConsumption * (part.mass / (part as SolidRocket).internalFuel);
                }

            return totalFuelConsumption;
        }

        // Get the DELTA-V of stage.
        public float GetStageDeltaV(int stage)
        {
            float totalDeltaV = 0;

            totalDeltaV = (GetStageVacIsp(stage) * 9.8068f) * Mathf.Log(GetStageTotalMass(stage) / (GetStageTotalMass(stage) - GetStageFuelMass(stage)));

            return totalDeltaV ;
        }
    }
}

